package kr.co.iya.base.config;

import javax.servlet.MultipartConfigElement;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.autoconfigure.condition.ConditionalOnWebApplication;
import org.springframework.boot.web.servlet.MultipartConfigFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.core.env.Environment;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.util.unit.DataSize;

import kr.co.iya.base.context.MaskContext;
import kr.co.iya.base.support.aid.AwareAidPack;
import kr.co.iya.base.support.aid.AwareAidPack.ApplicationContextAid;
import kr.co.iya.base.support.aid.BeanAidPack;
import kr.co.iya.base.support.aid.BeanAidPack.BeanAid;
import kr.co.iya.base.support.aid.RequestAttributesAidPack;
import kr.co.iya.base.support.aid.RequestAttributesAidPack.RequestAttributesAid;
import kr.co.iya.base.support.util.Base64UtilPack;
import kr.co.iya.base.support.util.Base64UtilPack.Base64Util;
import kr.co.iya.base.support.util.BashProcessUtilPack;
import kr.co.iya.base.support.util.BashProcessUtilPack.BashProcessUtil;
import kr.co.iya.base.support.util.CompressUtilPack;
import kr.co.iya.base.support.util.CompressUtilPack.CompressUtil;
import kr.co.iya.base.support.util.CryptoUtilPack;
import kr.co.iya.base.support.util.CryptoUtilPack.CryptoUtil;
import kr.co.iya.base.support.util.FileUtilPack;
import kr.co.iya.base.support.util.FileUtilPack.FileUtil;
import kr.co.iya.base.support.util.HttpRequestUtilPack;
import kr.co.iya.base.support.util.HttpRequestUtilPack.HttpRequestUtil;
import kr.co.iya.base.support.util.JsonUtilPack;
import kr.co.iya.base.support.util.JsonUtilPack.JsonUtil;
import kr.co.iya.base.support.util.LobConverterUtilPack;
import kr.co.iya.base.support.util.LobConverterUtilPack.LobConverterUtil;
import kr.co.iya.base.support.util.LocaleUtilPack;
import kr.co.iya.base.support.util.LocaleUtilPack.LocaleUtil;
import kr.co.iya.base.support.util.MaskUtilPack;
import kr.co.iya.base.support.util.MaskUtilPack.MaskUtil;
import kr.co.iya.base.support.util.MessageUtilPack;
import kr.co.iya.base.support.util.MessageUtilPack.MessageUtil;
import kr.co.iya.base.support.util.ObjectMapperUtilPack;
import kr.co.iya.base.support.util.ObjectMapperUtilPack.ObjectMapperUtil;
import kr.co.iya.base.support.util.PropertiesUtilPack;
import kr.co.iya.base.support.util.PropertiesUtilPack.PropertiesUtil;
import kr.co.iya.base.support.util.RestTemplateUtilPack;
import kr.co.iya.base.support.util.RestTemplateUtilPack.RestTemplateUtil;
import kr.co.iya.base.support.util.SessionUtilPack;
import kr.co.iya.base.support.util.SessionUtilPack.SessionUtil;
import kr.co.iya.base.support.util.SftpUtilPack;
import kr.co.iya.base.support.util.SftpUtilPack.SftpUtil;
import kr.co.iya.base.support.util.SystemUtilPack;
import kr.co.iya.base.support.util.SystemUtilPack.SystemUtil;
import kr.co.iya.base.support.util.ThreadUtilPack;
import kr.co.iya.base.support.util.ThreadUtilPack.ThreadUtil;
import kr.co.iya.base.support.util.TimeUtilPack;
import kr.co.iya.base.support.util.TimeUtilPack.TimeUtil;
import kr.co.iya.base.support.util.TransactionUtilPack;
import kr.co.iya.base.support.util.TransactionUtilPack.TransactionUtil;

@Configuration
public class AidBaseConfig {

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	@Value("${spring.application.name}")
	private String applicationName;

	@Value("${server.name:}")
	private String serverName;	
	
	@Value("${server.port:}")
	private String serverPort;

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	private final Environment environment;

	private final ReloadableResourceBundleMessageSource messageSource;

	public AidBaseConfig(Environment environment, ReloadableResourceBundleMessageSource messageSource) {
		this.environment = environment;
		this.messageSource = messageSource;
	}

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    @Bean
    @ConditionalOnExpression("'${spring.main.web-application-type:true}'.equals('none')")
    MultipartConfigElement getMultipartConfigElement() {    	
    	MultipartConfigFactory factory = new MultipartConfigFactory();
    	factory.setMaxFileSize(DataSize.ofMegabytes(10));
    	return factory.createMultipartConfig();
    }

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    @Bean
    ApplicationContextAid getApplicationContextAid() {
		return AwareAidPack.getApplicationContextAid();
	}

    @Bean
    BeanAid getBeanAid() {
		return BeanAidPack.getBeanAid();
	}

    @Bean
    RequestAttributesAid getRequestAttributesAid() {
		return RequestAttributesAidPack.getRequestAttributesAid();
	}

    @Bean
    Base64Util getBase64Util() {
		return Base64UtilPack.getBase64Util();
	}

    @Bean
    BashProcessUtil getBashProcessUtil() {
		return BashProcessUtilPack.getBashProcessUtil();
	}
    

    @Bean
    CompressUtil getCompressUtil() {
    	return CompressUtilPack.getCompressUtil();
    }
    
    @Bean
    CryptoUtil getCryptoUtil() {
		return CryptoUtilPack.getCryptoUtil();
	}
    
    @Bean
    FileUtil getFileUtil() {
		return FileUtilPack.getFileUtil(getSystemUtil());	
	}

    @Bean
    @ConditionalOnWebApplication
    HttpRequestUtil getHttpRequestUtil() {
		return HttpRequestUtilPack.getHttpRequestUtil(getRequestAttributesAid());
	}

    @Bean
    JsonUtil getJsonUtil() {
		return JsonUtilPack.getJsonUtil(true);
	}


    @Bean
    LobConverterUtil getLobConverterUtil() {
		return LobConverterUtilPack.getLobConverterUtil(getFileUtil());
	}

    @Bean
    LocaleUtil getLocaleUtil() {
		return LocaleUtilPack.getLocaleUtil(getRequestAttributesAid());
	}

    @Bean
    MaskUtil getMaskUtil() {
    	MaskContext context = null;
    	try{context = ApplicationContextAid.getApplicationContext().getBean(MaskContext.class);}catch(Exception e) {}
		return MaskUtilPack.getMaskUtil(context);
	}

    @Bean
    MessageUtil getMessageUtil() {
		return MessageUtilPack.getMessageUtil(getLocaleUtil(),this.messageSource);
	}

    @Bean
    ObjectMapperUtil getObjectMapperUtil() {
		return ObjectMapperUtilPack.getObjectMapperUtil();
	}

    @Bean
    @ConditionalOnWebApplication
    SessionUtil getSessionUtil() {
		return SessionUtilPack.getSessionUtil(getRequestAttributesAid());
	}

    @Bean
    SftpUtil getSftpUtil() {
		return SftpUtilPack.getSftpUtil(getFileUtil());
    }

    @Bean
    SystemUtil getSystemUtil() {
		return SystemUtilPack.getSystemUtil(this.environment,this.serverName,this.serverPort,this.applicationName);
	}
    
    @Bean
    ThreadUtil getThreadUtil() {
		return ThreadUtilPack.getThreadUtil();
	}
    
    @Bean
    TimeUtil getTimeUtil() {
		return TimeUtilPack.getTimeUtil();
	}

    @Bean
    RestTemplateUtil getRestTemplateUtil() {
		return RestTemplateUtilPack.getRestTemplateUtil();
	}

    @Bean
    PropertiesUtil getPropertiesUtil() {
		return PropertiesUtilPack.getPropertiesUtil(getBeanAid(),getJsonUtil(),PropertiesConfig.DEFAULT_APPLICATION_PROPS);
	}

    @Bean
    @ConditionalOnWebApplication
    TransactionUtil getTransactionUtil() {
    	PlatformTransactionManager txManager = null;
    	try{txManager = ApplicationContextAid.getApplicationContext().getBean(PlatformTransactionManager.class);}catch(Exception e) {}
		return TransactionUtilPack.getTransactionUtil(txManager);
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}