package kr.co.iya.base.config;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.jdbc.datasource.LazyConnectionDataSourceProxy;
import org.springframework.transaction.PlatformTransactionManager;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Configuration
@ConditionalOnProperty(prefix="spring.datasource.hikari",name="jdbc-url")
@ConditionalOnExpression("T(kr.co.iya.base.DatasourceConfig).isEnabled()")
public class DataSourceConfig {

    @Bean(name="default.hikariConfig")
    @ConfigurationProperties(prefix="spring.datasource.hikari")
    HikariConfig hikariConfig() {
    	log.debug(">> default.hikariConfig");
		return new HikariConfig();
	}

    @Bean(name="default.dataSource")
    @ConditionalOnBean(name="default.hikariConfig")
    LazyConnectionDataSourceProxy dataSource(@Qualifier("default.hikariConfig") HikariConfig hikariConfig) {
    	log.debug(">> default.dataSource");
    	//스프링의 경우 트랜잭션 시작 시, datasource 의 connection 를 사용하건 안하건 커넥션을 확보하며, 그로 인해 불필요한 리소스가 발생됨.
    	//이를 줄이기 위해 LazyConnectionDataSourceProxy로 wrapping 할 경우 실제 커넥션이 필요한 경우에만 datasource 에서 connection 을 반환    	
		DataSource dataSource = new HikariDataSource(hikariConfig);
		return new LazyConnectionDataSourceProxy(dataSource);
	}

    @Bean(name="default.txManager")
    @ConditionalOnBean(name="default.dataSource")
    PlatformTransactionManager txManager(@Qualifier("default.dataSource") DataSource dataSource) {
    	log.debug(">> default.txManager");
		DataSourceTransactionManager txManager = new DataSourceTransactionManager(dataSource);
		txManager.setNestedTransactionAllowed(true);
		txManager.setGlobalRollbackOnParticipationFailure(false);
		return txManager;
	}

}