package kr.co.iya.base.config;

import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.stereotype.Component;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Configuration
@ConditionalOnExpression("T(kr.co.iya.base.MessageSourceConfig).isEnabled()")
public class MessageSourceConfig {

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	@Component
	@ConfigurationProperties(prefix="utcb.messagesource")
	public static class MessageSourceProperties{
		private final String defaultMessageSource="classpath:message/message-default";
		
		private Set<String> baseNames;

		public Set<String> getBaseNames() {return baseNames;}
		public void setBaseNames(Set<String> baseNames) {this.baseNames = baseNames;}
		
		public String getDefaultmessagesource() {return defaultMessageSource;}
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	@Autowired
	private MessageSourceProperties props;
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    @Bean
    ReloadableResourceBundleMessageSource messageSource() {
		ReloadableResourceBundleMessageSource source = new ReloadableResourceBundleMessageSource();
		source.setBasenames(this.getBaseNames());
        source.setDefaultEncoding("UTF-8");
        source.setCacheSeconds(60);
        source.setUseCodeAsDefaultMessage(true);
        return source;
    }

	private String[] getBaseNames() {
		Set<String> baseNames = this.props.getBaseNames();		
		if(baseNames==null) {baseNames = new HashSet<>();}
		baseNames.add(props.getDefaultmessagesource());		
		baseNames.forEach(i->log.info(">> messageSource: {}",i));
		return baseNames.toArray(new String[baseNames.size()]);
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
}


