package kr.co.iya.base.config;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.Set;

import org.springframework.beans.factory.config.YamlPropertiesFactoryBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.core.env.PropertiesPropertySource;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.support.EncodedResource;
import org.springframework.core.io.support.PropertySourceFactory;
import org.springframework.lang.Nullable;

import kr.co.iya.base.config.PropertiesConfig.YamlPropertySourceFactory;

@Configuration
@ConditionalOnExpression("T(kr.co.iya.base.PropertiesConfig).isEnabled()")
@PropertySource(value="classpath:application.yaml", factory=YamlPropertySourceFactory.class)
public class PropertiesConfig implements EnvironmentAware {

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public static final String DEFAULT_APPLICATION_PROPS = "applicationProps";
	
	private Properties applicationProps;
	
	@Override
	public void setEnvironment(Environment environment) {
		YamlPropertiesFactoryBean factory = new YamlPropertiesFactoryBean();
		factory.setResources(new ClassPathResource("application.yaml")); 
		factory.afterPropertiesSet();
		Properties props = factory.getObject();
		
		Set<String> keys = props.stringPropertyNames();
		for (String key : keys) {
			props.put(key, environment.getProperty(key));
		}
		
		this.applicationProps = props;	    
	 }

    @Bean(name=DEFAULT_APPLICATION_PROPS)
    Properties getApplicationProps() {
		 return this.applicationProps;
	 }

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	 
	public static class YamlPropertySourceFactory implements PropertySourceFactory {

	    @Override
	    public org.springframework.core.env.PropertySource<?> createPropertySource(@Nullable String name, EncodedResource resource) throws IOException {
	        String sourceName = name!= null?name:resource.getResource().getFilename();
	        Properties properties = this.loadYamlIntoProperties(resource);
	        return new PropertiesPropertySource(sourceName,properties);
	    }

	    private Properties loadYamlIntoProperties(EncodedResource resource) throws FileNotFoundException {
	        try {
	            YamlPropertiesFactoryBean factory = new YamlPropertiesFactoryBean();
	            factory.setResources(resource.getResource());
	            factory.afterPropertiesSet();
	            return factory.getObject();
	        } catch (IllegalStateException e) {
	            Throwable cause = e.getCause();
	            if (cause instanceof FileNotFoundException){throw (FileNotFoundException) e.getCause();}
	            throw e;
	        }
	    }
	} 
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}


