package kr.co.iya.base.config;

import java.util.Collections;

import org.springframework.aop.Advisor;
import org.springframework.aop.aspectj.AspectJExpressionPointcut;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.interceptor.MatchAlwaysTransactionAttributeSource;
import org.springframework.transaction.interceptor.RollbackRuleAttribute;
import org.springframework.transaction.interceptor.RuleBasedTransactionAttribute;
import org.springframework.transaction.interceptor.TransactionInterceptor;

@Configuration
@EnableTransactionManagement
@ConditionalOnBean(PlatformTransactionManager.class)
@ConditionalOnExpression("T(kr.co.iya.base.TransactionConfig).isEnabled()")
public class TransactionConfig {

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	private static final String AOP_TX_METHOD_NAME = "*";
	private static final String AOP_TX_EXPRESSION = "execution(* kr.co.iya..service..*Impl.*(..))";
	
	private final TransactionManager txManager;

	public TransactionConfig(TransactionManager transactionManager) {
		this.txManager = transactionManager;
	}

    @Bean
    TransactionInterceptor txInterceptor() {
		RuleBasedTransactionAttribute attribute = new RuleBasedTransactionAttribute();
		attribute.setName(AOP_TX_METHOD_NAME);
		attribute.setRollbackRules(Collections.singletonList(new RollbackRuleAttribute(Exception.class)));
		attribute.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);

		MatchAlwaysTransactionAttributeSource attributeSource = new MatchAlwaysTransactionAttributeSource  ();
		attributeSource.setTransactionAttribute(attribute);
		
		return new TransactionInterceptor(this.txManager,attributeSource);
	}

//	Spring-Boot 2.3 이전사용시
//	@Bean
//	public TransactionInterceptor transactionAdvice() {
//		RuleBasedTransactionAttribute attribute = new RuleBasedTransactionAttribute();
//		attribute.setRollbackRules(Collections.singletonList(new RollbackRuleAttribute(Exception.class)));
//		attribute.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
//
//		Map<String,TransactionAttribute> txMethods = new HashMap<>();
//		txMethods.put(AOP_TX_METHOD_NAME,attribute);
//
//		NameMatchTransactionAttributeSource attributeSource = new NameMatchTransactionAttributeSource();
//		attributeSource.setNameMap(txMethods);
//		
//		return new TransactionInterceptor(this.transactionManager,attributeSource);
//	}
    
    @Bean
    Advisor txAdvisor() {
		AspectJExpressionPointcut pointcut = new AspectJExpressionPointcut();
		pointcut.setExpression(AOP_TX_EXPRESSION);
		return new DefaultPointcutAdvisor(pointcut, txInterceptor());
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}