package kr.co.iya.base.support;

import java.util.Map;

import org.springframework.context.ApplicationContext;

import kr.co.iya.base.advice.ExceptionAdvice;
import kr.co.iya.base.support.aid.AwareAidPack.ApplicationContextAid;
import kr.co.iya.base.support.aid.AwareAidPack.ThreadLocalAid;
import kr.co.iya.base.support.util.Base64UtilPack.Base64Util;
import kr.co.iya.base.support.util.BashProcessUtilPack.BashProcessUtil;
import kr.co.iya.base.support.util.CompressUtilPack.CompressUtil;
import kr.co.iya.base.support.util.CryptoUtilPack.CryptoUtil;
import kr.co.iya.base.support.util.FileUtilPack.FileUtil;
import kr.co.iya.base.support.util.HttpRequestUtilPack.HttpRequestUtil;
import kr.co.iya.base.support.util.JsonUtilPack.JsonUtil;
import kr.co.iya.base.support.util.LobConverterUtilPack.LobConverterUtil;
import kr.co.iya.base.support.util.LocaleUtilPack.LocaleUtil;
import kr.co.iya.base.support.util.MaskUtilPack.MaskUtil;
import kr.co.iya.base.support.util.MessageUtilPack.MessageUtil;
import kr.co.iya.base.support.util.ObjectMapperUtilPack.ObjectMapperUtil;
import kr.co.iya.base.support.util.PropertiesUtilPack.PropertiesUtil;
import kr.co.iya.base.support.util.RestTemplateUtilPack.RestTemplateUtil;
import kr.co.iya.base.support.util.SessionUtilPack.SessionUtil;
import kr.co.iya.base.support.util.SftpUtilPack.SftpUtil;
import kr.co.iya.base.support.util.SystemUtilPack.SystemUtil;
import kr.co.iya.base.support.util.ThreadUtilPack.ThreadUtil;
import kr.co.iya.base.support.util.TimeUtilPack.TimeUtil;
import kr.co.iya.base.support.util.TransactionUtilPack.TransactionUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public abstract class AbstractUtil {

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static ApplicationContext getApplicationContext() {return ApplicationContextAid.getApplicationContext();}
	public static Map<String,Object> getThreadLocalMap() {return ThreadLocalAid.getThreadLocalMap();}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static <T> T getBean(Class<T> type) {
		try {
			return getApplicationContext().getBean(type);
		}catch(Exception e) {
			log.error("\n{}",ExceptionAdvice.getTrace(e));
			return null;
		}
	}
	
	public static <T> T getBean(String beanId, Class<T> type) {
		try {
			return getApplicationContext().getBean(beanId,type);
		}catch(Exception e) {
			log.error("\n{}",ExceptionAdvice.getTrace(e));
			return null;
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// kr.co.iya.base.config.AidBaseConfig에 정의된 bean에 대한 method 등록
	
	public static final Base64Util getBase64Util() {return getBean(Base64Util.class);}
	public static final BashProcessUtil getBashProcessUtil() {return getBean(BashProcessUtil.class);}
	public static final CompressUtil getCompressUtil() {return getBean(CompressUtil.class);} 
	public static final CryptoUtil getCryptoUtil() {return getBean(CryptoUtil.class);} 
	public static final FileUtil getFileUtil() { return getBean(FileUtil.class);} 	
	
	public static final HttpRequestUtil getHttpRequestUtil() {return getBean(HttpRequestUtil.class);}
	public static final JsonUtil getJsonUtil() {return getBean(JsonUtil.class);}
	public static final LobConverterUtil getLobConvertUtil() {return getBean(LobConverterUtil.class);}
	public static final LocaleUtil getLocaleUtil() {return getBean(LocaleUtil.class);} 
	public static final MaskUtil getMaskUtil() {return getBean(MaskUtil.class);} 	
	public static final MessageUtil getMessageUtil() {return getBean(MessageUtil.class);} 
	public static final ObjectMapperUtil getObjectMapperUtil() {return getBean(ObjectMapperUtil.class);}
	public static final PropertiesUtil getPropertiesUtil() {return getBean(PropertiesUtil.class);}
	public static final RestTemplateUtil getRestTemplateUtil() {return getBean(RestTemplateUtil.class);} 	
	public static final SessionUtil getSessionUtil() {return getBean(SessionUtil.class);} 	
	public static final SftpUtil getSftpUtil() {return getBean(SftpUtil.class);}
	public static final SystemUtil getSystemUtil() {return getBean(SystemUtil.class);}
	public static final ThreadUtil getThreadUtil() {return getBean(ThreadUtil.class);}
	public static final TimeUtil getTimeUtil() {return getBean(TimeUtil.class);} 
    public static final TransactionUtil getTransactionUtil() {return getBean(TransactionUtil.class);}
} 
